.ag-theme-uiwiz {
    --ag-border-color: var(--color-base-content);
    --ag-foreground-color: var(--color-base-content);
    --ag-background-color: var(--color-base-100);
    --ag-header-background-color: var(--color-base-300);
    --ag-row-hover-color: var(--color-base-200);
}